@@ -1,21 +1,10 @@
 /*
     knewstuff3/engine.h.
-    Copyright (c) 2007 Josef Spillner <spillner@kde.org>
-    Copyright (C) 2007-2010 Frederik Gladhorn <gladhorn@kde.org>
-    Copyright (c) 2009 Jeremy Whiting <jpwhiting@kde.org>
+    SPDX-FileCopyrightText: 2007 Josef Spillner <spillner@kde.org>
+    SPDX-FileCopyrightText: 2007-2010 Frederik Gladhorn <gladhorn@kde.org>
+    SPDX-FileCopyrightText: 2009 Jeremy Whiting <jpwhiting@kde.org>
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library.  If not, see <http://www.gnu.org/licenses/>.
+    SPDX-License-Identifier: LGPL-2.1-or-later
 */
 
 #ifndef KNEWSTUFF3_ENGINE_P_H
@@ -70,6 +59,26 @@ class Installation;
 class KNEWSTUFFCORE_EXPORT Engine : public QObject
 {
     Q_OBJECT
+
+    /**
+     * Current state of the engine, the state con contain multiple operations
+     * an empty BusyState represents the idle status
+     * @since 5.74
+     */
+    Q_PROPERTY(BusyState busyState READ busyState WRITE setBusyState NOTIFY busyStateChanged)
+
+    /**
+     * String representation of the engines busy state, in the case of idle this string is empty
+     * @since 5.74
+     */
+    Q_PROPERTY(QString busyMessage READ busyMessage WRITE setBusyMessage NOTIFY busyMessageChanged)
+
+    /**
+     * Text that should be displayed for the adoption button, this defaults to "Use"
+     * @since 5.77
+     */
+    Q_PROPERTY(QString useLabel READ useLabel NOTIFY useLabelChanged)
+
 public:
     /**
      * Constructor.
@@ -82,6 +91,14 @@ public:
      */
     ~Engine();
 
+    enum class BusyOperation {
+        Initializing,
+        LoadingData,
+        LoadingPreview,
+        InstallingEntry,
+    };
+    Q_DECLARE_FLAGS(BusyState, BusyOperation)
+
     /**
      * Initializes the engine. This step is application-specific and relies
      * on an external configuration file, which determines all the details
@@ -89,6 +106,7 @@ public:
      *
      * @param configfile KNewStuff2 configuration file (*.knsrc)
      * @return \b true if any valid configuration was found, \b false otherwise
+    * @see KNS3::DownloadDialog
      */
     bool init(const QString &configfile);
 
@@ -235,7 +253,7 @@ public:
      * tagname!=tagdata
      *
      * == KNSRC entry ==
-     * A tag filter line in a .knsrc file, which is a comma semarated list of
+     * A tag filter line in a .knsrc file, which is a comma separated list of
      * tag/value pairs, might look like:
      *
      * TagFilter=ghns_excluded!=1,data##mimetype==application/cbr+zip,data##mimetype==application/cbr+rar
@@ -250,7 +268,7 @@ public:
      * which do not mark themselves as having a 64bit application binary in at
      * least one download item.
      *
-     * The value does not current suppport wildcards. The list should be considered
+     * The value does not current support wildcards. The list should be considered
      * a binary AND operation (that is, all filter entries must match for the data
      * entry to be included in the return data)
      *
@@ -397,6 +415,7 @@ public:
      */
     QList<Provider::CategoryMetadata> categoriesMetadata();
 
+#if KNEWSTUFFCORE_ENABLE_DEPRECATED_SINCE(5, 77)
     /**
      * The adoption command can be used to allow a user to make use of an entry's
      * installed data. For example, this command might be used to ask the system to
@@ -412,8 +431,12 @@ public:
      *
      * @param entry The entry to return an adoption command for
      * @return The command to run to adopt this entry's installed data
+     * @deprecated Since 5.77, use Engine::adoptEntry(const KNSCore::EntryInternal &entry) instead
      */
+    KNEWSTUFFCORE_DEPRECATED_VERSION(5, 77, "Use Engine::adoptEntry(const KNSCore::EntryInternal &entry) instead")
     QString adoptionCommand(const KNSCore::EntryInternal &entry) const;
+#endif
+
     /**
      * Whether or not an adoption command exists for this engine
      *
@@ -422,6 +445,27 @@ public:
      */
     bool hasAdoptionCommand() const;
 
+    /**
+     * Adopt an entry using the adoption command. This will also take care of displaying error messages
+     * @param entry Entry that should be adopted
+     * @see signalErrorCode
+     * @see signalEntryEvent
+     * @since 5.77
+     */
+    Q_INVOKABLE void adoptEntry(const KNSCore::EntryInternal &entry);
+
+    /**
+     * Text that should be displayed for the adoption button, this defaults to i18n("Use")
+     * @since 5.77
+     */
+    QString useLabel() const;
+
+    /**
+     * Signal gets emitted when the useLabel property changes
+     * @since 5.77
+     */
+    Q_SIGNAL void useLabelChanged();
+
     /**
      * Set the page size for requests not made explicitly with requestData(int,int)
      * @param pageSize the default number of entries to request from the provider
@@ -477,6 +521,67 @@ public:
      */
     CommentsModel *commentsForEntry(const KNSCore::EntryInternal &entry);
 
+    /**
+     * String representation of the engines busy state
+     * @since 5.74
+     */
+    QString busyMessage() const;
+
+    /**
+     * @since 5.74
+     * @see setBusy
+     * @see setBusyState
+     */
+    void setBusyMessage(const QString &busyMessage);
+
+    /**
+     * Signal gets emitted when the busy message changes
+     * @since 5.74 String representation of the engines busy state
+     */
+    Q_SIGNAL void busyMessageChanged();
+
+    /**
+     * Busy state of the engine
+     * @since 5.74
+     */
+    BusyState busyState() const;
+
+    /**
+     * Sets the busy state of the engine
+     * @since 5.74
+     * @see setBusy
+     * @see setBusyMessage
+     */
+    void setBusyState(BusyState state);
+
+    /**
+     * Signal gets emitted when the busy state changes
+     * @since 5.74
+     */
+    Q_SIGNAL void busyStateChanged();
+
+    /**
+     * Utility method to set both the state and busyMessage
+     * @since 5.74
+     */
+    void setBusy(BusyState state, const QString &busyMessage);
+
+    /**
+     * Get the entries cache for this engine (note that it may be null if the engine is
+     * not yet initialized).
+     * @return The cache for this engine (or null if the engine is not initialized)
+     * @since 5.74
+     */
+    QSharedPointer<Cache> cache() const;
+
+    /**
+     * If the same engine gets reused and the user could have used the delete functionality of the KCMs the cache could
+     * be out of sync. If the RemoveDeadEntries option is set to true this will remove deleted entries from the cache
+     * and the signalEntryChanged slot will be emitted with the updated entry
+     * @since 5.74
+     */
+    Q_INVOKABLE void revalidateCacheEntries();
+
 Q_SIGNALS:
     /**
      * Indicates a message to be added to the ui's log, or sent to a messagebox
@@ -486,8 +591,16 @@ Q_SIGNALS:
     void signalProvidersLoaded();
     void signalEntriesLoaded(const KNSCore::EntryInternal::List &entries);
     void signalUpdateableEntriesLoaded(const KNSCore::EntryInternal::List &entries);
+
+#if KNEWSTUFFCORE_ENABLE_DEPRECATED_SINCE(5, 77)
+    KNEWSTUFFCORE_DEPRECATED_VERSION(5, 77, "Use Engine::signalEntryEvent instead")
     void signalEntryChanged(const KNSCore::EntryInternal &entry);
+#endif
+
+#if KNEWSTUFFCORE_ENABLE_DEPRECATED_SINCE(5, 77)
+    KNEWSTUFFCORE_DEPRECATED_VERSION(5, 77, "Use Engine::signalEntryEvent instead")
     void signalEntryDetailsLoaded(const KNSCore::EntryInternal &entry);
+#endif
 
     // a new search result is there, clear the list of items
     void signalResetView();
@@ -501,9 +614,19 @@ Q_SIGNALS:
     void signalDownloadDialogDone(KNSCore::EntryInternal::List);
     void jobStarted(KJob *, const QString &);
 
-    QT_DEPRECATED void signalError(const QString &);
+#if KNEWSTUFFCORE_ENABLE_DEPRECATED_SINCE(5, 53)
+    KNEWSTUFFCORE_DEPRECATED_VERSION(5, 53, "Use Engine::signalErrorCode(const KNSCore::ErrorCode &, const QString &, const QVariant &)")
+    void signalError(const QString &);
+#endif
+#if KNEWSTUFFCORE_ENABLE_DEPRECATED_SINCE(5, 74)
+        KNEWSTUFFCORE_DEPRECATED_VERSION(5, 74, "Use Engine::busyStateChanged() and Engine::busyMessageChanged() instead")
     void signalBusy(const QString &);
+#endif
+#if KNEWSTUFFCORE_ENABLE_DEPRECATED_SINCE(5, 74)
+        KNEWSTUFFCORE_DEPRECATED_VERSION(5, 74, "Use Engine::busyStateChanged() and Engine::busyMessageChanged() instead")
     void signalIdle(const QString &);
+#endif
+
     /**
      * Fires in the case of any critical or serious errors, such as network or API problems.
      * @param errorCode Represents the specific type of error which has occurred
@@ -516,6 +639,8 @@ Q_SIGNALS:
 
     void signalCategoriesMetadataLoded(const QList<Provider::CategoryMetadata> &categories);
 
+    void signalEntryEvent(const EntryInternal &entry, EntryInternal::EntryEvent event);
+
 private Q_SLOTS:
     // the .knsrc file was loaded
     void slotProviderFileLoaded(const QDomDocument &doc);
