@@ -1,34 +1,16 @@
 /*
- * Copyright (C) 2016 Dan Leinir Turthra Jensen <admin@leinir.dk>
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) version 3, or any
- * later version accepted by the membership of KDE e.V. (or its
- * successor approved by the membership of KDE e.V.), which shall
- * act as a proxy defined in Section 6 of version 3 of the license.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library.  If not, see <http://www.gnu.org/licenses/>.
- *
- */
+    SPDX-FileCopyrightText: 2016 Dan Leinir Turthra Jensen <admin@leinir.dk>
+
+    SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
+*/
 
 #include "quickitemsmodel.h"
 #include "quickengine.h"
-#include "knewstuffquick_debug.h"
-
 #include "itemsmodel.h"
 #include "engine.h"
 #include "downloadlinkinfo.h"
 #include "core/commentsmodel.h"
 
-#include <KLocalizedString>
 #include <KShell>
 #include <QProcess>
 
@@ -61,20 +43,32 @@ public:
             return false;
         }
         model = new KNSCore::ItemsModel(coreEngine, q);
-
-        q->connect(coreEngine, &KNSCore::Engine::signalBusy, q, [=](){ isLoadingData = true; emit q->isLoadingDataChanged(); });
-        q->connect(coreEngine, &KNSCore::Engine::signalIdle, q, [=](){ isLoadingData = false; emit q->isLoadingDataChanged(); });
+        q->connect(coreEngine, &KNSCore::Engine::busyStateChanged, q, [=](){
+            // If we install/update an entry the spinner should be hidden, BUG: 422047
+            const KNSCore::Engine::BusyState state = coreEngine->busyState();
+            const bool busy = state && !state.testFlag(KNSCore::Engine::BusyOperation::InstallingEntry);
+            if (isLoadingData != busy) {
+                isLoadingData = busy;
+                Q_EMIT q->isLoadingDataChanged();
+            }
+        });
 
         q->connect(coreEngine, &KNSCore::Engine::signalProvidersLoaded, coreEngine, &KNSCore::Engine::reloadEntries);
         // Entries have been fetched and should be shown:
-        q->connect(coreEngine, &KNSCore::Engine::signalEntriesLoaded, model, &KNSCore::ItemsModel::slotEntriesLoaded);
-
-        // An entry has changes - eg because it was installed
-        q->connect(coreEngine, &KNSCore::Engine::signalEntryChanged, model, &KNSCore::ItemsModel::slotEntryChanged);
-        q->connect(coreEngine, &KNSCore::Engine::signalEntryChanged, q, [=](const KNSCore::EntryInternal &entry){
-            emit q->entryChanged(model->row(entry));
+        q->connect(coreEngine, &KNSCore::Engine::signalEntriesLoaded, model, [this](const KNSCore::EntryInternal::List& entries){
+            if (coreEngine->filter() != KNSCore::Provider::Updates) {
+                model->slotEntriesLoaded(entries);
+            }
+        });
+        q->connect(coreEngine, &KNSCore::Engine::signalUpdateableEntriesLoaded, model, [this](const KNSCore::EntryInternal::List& entries){
+            if (coreEngine->filter() == KNSCore::Provider::Updates) {
+                model->slotEntriesLoaded(entries);
+            }
         });
 
+        q->connect(coreEngine, &KNSCore::Engine::signalEntryEvent, q, [this](const KNSCore::EntryInternal &entry, KNSCore::EntryInternal::EntryEvent event) {
+            onEntryEvent(entry, event);
+        });
         q->connect(coreEngine, &KNSCore::Engine::signalResetView, model, &KNSCore::ItemsModel::clearEntries);
         q->connect(coreEngine, &KNSCore::Engine::signalEntryPreviewLoaded, model, &KNSCore::ItemsModel::slotEntryPreviewLoaded);
 
@@ -84,6 +78,22 @@ public:
         q->connect(model, &KNSCore::ItemsModel::modelReset, q, &ItemsModel::modelReset);
         return true;
     }
+
+    void onEntryEvent(const KNSCore::EntryInternal &entry, KNSCore::EntryInternal::EntryEvent event)
+    {
+        if (event == KNSCore::EntryInternal::StatusChangedEvent) {
+            model->slotEntryChanged(entry);
+            Q_EMIT q->entryChanged(model->row(entry));
+
+            // If we update/uninstall an entry we have to update the UI, see BUG: 425135
+            if (coreEngine->filter() == KNSCore::Provider::Updates
+                && entry.status() != KNS3::Entry::Updateable && entry.status() != KNS3::Entry::Updating) {
+                model->removeEntry(entry);
+            } else if (coreEngine->filter() == KNSCore::Provider::Installed && entry.status() == KNS3::Entry::Deleted) {
+                model->removeEntry(entry);
+            }
+        }
+    }
 };
 
 ItemsModel::ItemsModel(QObject *parent)
@@ -349,10 +359,7 @@ QVariant ItemsModel::data(const QModelIndex &index, int role) const
 
 bool ItemsModel::canFetchMore(const QModelIndex &parent) const
 {
-    if (!parent.isValid() && d->coreEngine && d->coreEngine->categoriesMetadata().count() > 0) {
-        return true;
-    }
-    return false;
+    return !parent.isValid() && d->coreEngine && d->coreEngine->categoriesMetadata().count() > 0;
 }
 
 void ItemsModel::fetchMore(const QModelIndex &parent)
@@ -386,7 +393,7 @@ void ItemsModel::setEngine(QObject *newEngine)
             d->coreEngine = qobject_cast<KNSCore::Engine*>(d->engine->engine());
             endResetModel();
         });
-        emit engineChanged();
+        Q_EMIT engineChanged();
         endResetModel();
     }
 }
@@ -406,6 +413,11 @@ void ItemsModel::installItem(int index, int linkId)
     }
 }
 
+void ItemsModel::updateItem(int index)
+{
+    installItem(index, AutoDetectLinkId);
+}
+
 void ItemsModel::uninstallItem(int index)
 {
     if (d->coreEngine) {
@@ -421,10 +433,7 @@ void ItemsModel::adoptItem(int index)
     if (d->coreEngine) {
         KNSCore::EntryInternal entry = d->model->data(d->model->index(index), Qt::UserRole).value<KNSCore::EntryInternal>();
         if (entry.isValid()) {
-            QStringList args = KShell::splitArgs(d->coreEngine->adoptionCommand(entry));
-            qCDebug(KNEWSTUFFQUICK) << "executing AdoptionCommand" << args;
-            QProcess::startDetached(args.takeFirst(), args);
-            d->engine->idleMessage(i18n("Using %1", entry.name()));
+            d->coreEngine->adoptEntry(entry);
         }
     }
 }
